@@ -1,3 +1,4 @@
+use crate::error::ErrorCode;
 use crate::{
     AccountDeserialize, AccountSerialize, Accounts, AccountsExit, CpiAccount, ToAccountInfo,
     ToAccountInfos, ToAccountMetas,
@@ -9,7 +10,7 @@ use solana_program::program_error::ProgramError;
 use solana_program::pubkey::Pubkey;
 use std::ops::{Deref, DerefMut};
 
-pub const PROGRAM_STATE_SEED: &'static str = "unversioned";
+pub const PROGRAM_STATE_SEED: &str = "unversioned";
 
 /// Boxed container for the program state singleton.
 #[derive(Clone)]
@@ -57,22 +58,23 @@ where
     fn try_accounts(
         program_id: &Pubkey,
         accounts: &mut &[AccountInfo<'info>],
+        _ix_data: &[u8],
     ) -> Result<Self, ProgramError> {
         if accounts.is_empty() {
-            return Err(ProgramError::NotEnoughAccountKeys);
+            return Err(ErrorCode::AccountNotEnoughKeys.into());
         }
         let account = &accounts[0];
         *accounts = &accounts[1..];
 
         if account.key != &Self::address(program_id) {
             solana_program::msg!("Invalid state address");
-            return Err(ProgramError::Custom(1)); // todo: proper error.
+            return Err(ErrorCode::StateInvalidAddress.into());
         }
 
         let pa = ProgramState::try_from(account)?;
         if pa.inner.info.owner != program_id {
             solana_program::msg!("Invalid state owner");
-            return Err(ProgramError::Custom(1)); // todo: proper error.
+            return Err(ErrorCode::AccountNotProgramOwned.into());
         }
         Ok(pa)
     }
